@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2005 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -13,8 +13,12 @@
 
 #include "collectionfieldsdialog.h"
 #include "collection.h"
+#include "field.h"
 #include "collectionfactory.h"
-#include "stringmapdialog.h"
+#include "gui/stringmapdialog.h"
+#include "tellico_kernel.h"
+#include "translators/tellico_xml.h"
+#include "tellico_utils.h"
 
 #include <klocale.h>
 #include <kcombobox.h>
@@ -23,6 +27,8 @@
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kpushbutton.h>
+#include <kaccelmanager.h>
+#include <kdeversion.h>
 
 #include <qgroupbox.h>
 #include <qlayout.h>
@@ -81,7 +87,6 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
     : KDialogBase(parent_, name_, false, i18n("Collection Fields"), Help|Default|Ok|Apply|Cancel, Ok, false),
       m_coll(coll_),
       m_defaultCollection(0),
-      m_typeMap(Data::Field::typeMap()),
       m_currentField(0),
       m_modified(false),
       m_updatingValues(false),
@@ -96,28 +101,29 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   m_fieldsBox = new QListBox(fieldsGroup);
   m_fieldsBox->setMinimumWidth(150);
 
-  for(Data::FieldListIterator it(m_coll->fieldList()); it.current(); ++it) {
+  Data::FieldVec fields = m_coll->fields();
+  for(Data::FieldVec::Iterator it = fields.begin(); it != fields.end(); ++it) {
     // ignore ReadOnly
-    if(it.current()->type() != Data::Field::ReadOnly) {
-      (void) new ListBoxText(m_fieldsBox, it.current());
+    if(it->type() != Data::Field::ReadOnly) {
+      (void) new ListBoxText(m_fieldsBox, it);
     }
   }
   connect(m_fieldsBox, SIGNAL(highlighted(int)), SLOT(slotHighlightedChanged(int)));
 
   QHBox* hb1 = new QHBox(fieldsGroup);
-  hb1->setSpacing(4);
-  m_btnNew = new KPushButton(i18n("New Field", "New"), hb1);
+  hb1->setSpacing(KDialog::spacingHint());
+  m_btnNew = new KPushButton(i18n("New Field", "&New"), hb1);
   m_btnNew->setIconSet(BarIcon(QString::fromLatin1("filenew"), KIcon::SizeSmall));
   QWhatsThis::add(m_btnNew, i18n("Add a new field to the collection"));
-  m_btnDelete = new KPushButton(i18n("Delete Field", "Delete"), hb1);
-  m_btnDelete->setIconSet(BarIcon(QString::fromLatin1("editdelete"), KIcon::SizeSmall));
+  m_btnDelete = new KPushButton(i18n("Delete Field", "&Delete"), hb1);
+  m_btnDelete->setIconSet(BarIconSet(QString::fromLatin1("editdelete"), KIcon::SizeSmall));
   QWhatsThis::add(m_btnDelete, i18n("Remove a field from the collection"));
 
   connect(m_btnNew, SIGNAL(clicked()), SLOT(slotNew()) );
   connect(m_btnDelete, SIGNAL(clicked()), SLOT(slotDelete()));
 
   QHBox* hb2 = new QHBox(fieldsGroup);
-  hb2->setSpacing(4);
+  hb2->setSpacing(KDialog::spacingHint());
   m_btnUp = new KPushButton(hb2);
   m_btnUp->setPixmap(BarIcon(QString::fromLatin1("up"), KIcon::SizeSmall));
   QWhatsThis::add(m_btnUp, i18n("Move this field up in the list. The list order is important "
@@ -140,19 +146,21 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   // (parent, nrows, ncols, margin, spacing)
   QGridLayout* layout = new QGridLayout(grid, 4, 4, 0, KDialog::spacingHint());
 
-  QLabel* label = new QLabel(i18n("Title:"), grid);
+  QLabel* label = new QLabel(i18n("&Title:"), grid);
   layout->addWidget(label, 0, 0);
   m_titleEdit = new KLineEdit(grid);
   layout->addWidget(m_titleEdit, 0, 1);
+  label->setBuddy(m_titleEdit);
   QString whats = i18n("The title of the field");
   QWhatsThis::add(label, whats);
   QWhatsThis::add(m_titleEdit, whats);
   connect(m_titleEdit, SIGNAL(textChanged(const QString&)), SLOT(slotModified()));
 
-  label = new QLabel(i18n("Type:"), grid);
+  label = new QLabel(i18n("T&ype:"), grid);
   layout->addWidget(label, 0, 2);
   m_typeCombo = new KComboBox(grid);
   layout->addWidget(m_typeCombo, 0, 3);
+  label->setBuddy(m_typeCombo);
   whats = QString::fromLatin1("<qt>");
   whats += i18n("The type of the field determines what values may be used. ");
   whats += i18n("<i>Simple Text</i> is used for most fields. ");
@@ -161,24 +169,26 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   whats += i18n("<i>Checkbox</i> is for a simple yes/no value. ");
   whats += i18n("<i>Number</i> indicates that the field contains a numerical value. ");
   whats += i18n("<i>URL</i> is for fields which refer to URLs, including references to other files. ");
-  whats += i18n("<i>Table</i>s may be a single or double column of values. ");
+  whats += i18n("A <i>Table</i> may hold one or more columns of values. ");
   whats += i18n("An <i>Image</i> field holds a picture. ");
   whats += i18n("A <i>Date</i> field can be used for values with a day, month, and year. ");
+  whats += i18n("A <i>Rating</i> field uses stars to show a rating number. ");
   whats += i18n("A <i>Dependent</i> field depends on the values of other "
                 "fields, and is formatted according to the field description. ");
   whats += i18n("A <i>Read Only</i> is for internal values, possibly useful for import and export. ");
   whats += QString::fromLatin1("</qt>");
   QWhatsThis::add(label, whats);
   QWhatsThis::add(m_typeCombo, whats);
-  // the typeTitles match the typeMap().values() but in a better order
+  // the typeTitles match the fieldMap().values() but in a better order
   m_typeCombo->insertStringList(Data::Field::typeTitles());
   connect(m_typeCombo, SIGNAL(activated(int)), SLOT(slotModified()));
   connect(m_typeCombo, SIGNAL(activated(const QString&)), SLOT(slotTypeChanged(const QString&)));
 
-  label = new QLabel(i18n("Category:"), grid);
+  label = new QLabel(i18n("Cate&gory:"), grid);
   layout->addWidget(label, 1, 0);
   m_catCombo = new KComboBox(true, grid);
   layout->addWidget(m_catCombo, 1, 1);
+  label->setBuddy(m_catCombo);
   whats = i18n("The field category determines where the field is placed in the editor.");
   QWhatsThis::add(label, whats);
   QWhatsThis::add(m_catCombo, whats);
@@ -187,8 +197,8 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   QStringList cats;
   const QStringList allCats = m_coll->fieldCategories();
   for(QStringList::ConstIterator it = allCats.begin(); it != allCats.end(); ++it) {
-    Data::FieldList list = m_coll->fieldsByCategory(*it);
-    if(!list.isEmpty() && !list.getFirst()->isSingleCategory()) {
+    Data::FieldVec fields = m_coll->fieldsByCategory(*it);
+    if(!fields.isEmpty() && !fields.begin()->isSingleCategory()) {
       cats.append(*it);
     }
   }
@@ -196,21 +206,12 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   m_catCombo->setDuplicatesEnabled(false);
   connect(m_catCombo, SIGNAL(textChanged(const QString&)), SLOT(slotModified()));
 
-  label = new QLabel(i18n("Description:"), grid);
-  layout->addWidget(label, 3, 0);
-  m_allowEdit = new KLineEdit(grid);
-  layout->addMultiCellWidget(m_allowEdit, 2, 2, 1, 3);
-  whats = i18n("<qt>For <i>Choice</i>-type fields, these are the only values allowed. They are "
-               "placed in a combo box.</qt>");
-  QWhatsThis::add(label, whats);
-  QWhatsThis::add(m_allowEdit, whats);
-  connect(m_allowEdit, SIGNAL(textChanged(const QString&)), SLOT(slotModified()));
-
-  label = new QLabel(i18n("Allowed:"), grid);
+  label = new QLabel(i18n("Descr&iption:"), grid);
   layout->addWidget(label, 2, 0);
   m_descEdit = new KLineEdit(grid);
   m_descEdit->setMinimumWidth(150);
-  layout->addMultiCellWidget(m_descEdit, 3, 3, 1, 3);
+  layout->addMultiCellWidget(m_descEdit, 2, 2, 1, 3);
+  label->setBuddy(m_descEdit);
   whats = i18n("The description is a useful reminder of what information is contained in the "
                "field. For <i>Dependent</i> fields, the description is a format string such as "
                "\"%{year} %{title}\" where the named fields get substituted in the string.");
@@ -218,12 +219,26 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
   QWhatsThis::add(m_descEdit, whats);
   connect(m_descEdit, SIGNAL(textChanged(const QString&)), SLOT(slotModified()));
 
-  label = new QLabel(i18n("Extended Properties:"), grid);
+  label = new QLabel(i18n("A&llowed:"), grid);
+  layout->addWidget(label, 3, 0);
+  m_allowEdit = new KLineEdit(grid);
+  layout->addMultiCellWidget(m_allowEdit, 3, 3, 1, 3);
+  label->setBuddy(m_allowEdit);
+  whats = i18n("<qt>For <i>Choice</i>-type fields, these are the only values allowed. They are "
+               "placed in a combo box.</qt>");
+  QWhatsThis::add(label, whats);
+  QWhatsThis::add(m_allowEdit, whats);
+  connect(m_allowEdit, SIGNAL(textChanged(const QString&)), SLOT(slotModified()));
+
+  label = new QLabel(i18n("Extended &properties:"), grid);
   layout->addWidget(label, 4, 0);
-  m_btnExtended = new KPushButton(i18n("Set..."), grid);
+  m_btnExtended = new KPushButton(i18n("&Set..."), grid);
+  m_btnExtended->setIconSet(BarIcon(QString::fromLatin1("bookmark"), KIcon::SizeSmall));
   layout->addMultiCellWidget(m_btnExtended, 4, 4, 1, 1);
+  label->setBuddy(m_btnExtended);
   whats = i18n("Extended field properties are used to specify things such as the corresponding bibtex field.");
   QWhatsThis::add(label, whats);
+  QWhatsThis::add(m_btnExtended, whats);
   connect(m_btnExtended, SIGNAL(clicked()), SLOT(slotShowExtendedProperties()));
 
   QButtonGroup* bg = new QButtonGroup(1, Qt::Horizontal, i18n("Format Options"), vbox);
@@ -257,6 +272,9 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
 
   // need to stretch at bottom
   vbox->setStretchFactor(new QWidget(vbox), 1);
+#if KDE_IS_VERSION(3,1,90)
+  KAcceleratorManager::manage(vbox);
+#endif
 
   // keep a default collection
   m_defaultCollection = CollectionFactory::collection(m_coll->type(), true);
@@ -278,8 +296,6 @@ CollectionFieldsDialog::CollectionFieldsDialog(Data::Collection* coll_, QWidget*
 }
 
 CollectionFieldsDialog::~CollectionFieldsDialog() {
-  delete m_defaultCollection;
-  m_defaultCollection = 0;
 }
 
 void CollectionFieldsDialog::slotSelectInitial() {
@@ -302,11 +318,16 @@ void CollectionFieldsDialog::slotApply() {
 
   updateField();
 
-  for(Data::FieldListIterator it(m_copiedFields); it.current(); ++it) {
+// start a command group, "Modify" is a generic term here since the commands could be add, modify, or delete
+  Kernel::self()->beginCommandGroup(i18n("Modify Fields"));
+
+  Data::FieldPtr field;
+  for(Data::FieldVec::Iterator it = m_copiedFields.begin(); it != m_copiedFields.end(); ++it) {
+    field = it;
     // check for Choice fields with removed values to warn user
-    if(it.current()->type() == Data::Field::Choice) {
-      QStringList oldValues = m_coll->fieldByName(it.current()->name())->allowed();
-      QStringList newValues = it.current()->allowed();
+    if(field->type() == Data::Field::Choice || field->type() == Data::Field::Rating) {
+      QStringList oldValues = m_coll->fieldByName(field->name())->allowed();
+      QStringList newValues = field->allowed();
       for(QStringList::ConstIterator vIt = oldValues.begin(); vIt != oldValues.end(); ++vIt) {
         if(newValues.contains(*vIt)) {
           continue;
@@ -315,51 +336,67 @@ void CollectionFieldsDialog::slotApply() {
                                                      i18n("<qt>Removing allowed values from the <i>%1</i> field which "
                                                           "currently exist in the collection may cause data corruption. "
                                                           "Do you want to keep your modified values or cancel and revert "
-                                                          "to the current ones?</qt>").arg(it.current()->title()),
+                                                          "to the current ones?</qt>").arg(field->title()),
                                                      QString::null,
                                                      i18n("Keep modified values"));
         if(ret != KMessageBox::Continue) {
-          it.current()->setAllowed(oldValues);
+          if(field->type() == Data::Field::Choice) {
+            field->setAllowed(oldValues);
+          } else { // rating field
+            Data::Field* oldField = m_coll->fieldByName(field->name());
+            field->setProperty(QString::fromLatin1("minimum"), oldField->property(QString::fromLatin1("minimum")));
+            field->setProperty(QString::fromLatin1("maximum"), oldField->property(QString::fromLatin1("maximum")));
+          }
         }
         break;
       }
     }
-    m_coll->modifyField(it.current());
+    Kernel::self()->modifyField(field);
   }
 
-
-  for(Data::FieldListIterator it(m_newFields); it.current(); ++it) {
-    m_coll->addField(it.current());
+  for(Data::FieldVec::Iterator it = m_newFields.begin(); it != m_newFields.end(); ++it) {
+    Kernel::self()->addField(it);
   }
 
   // set all text not to be colored, and get new list
-  Data::FieldList list;
+  Data::FieldVec fields;
   for(QListBoxItem* item = m_fieldsBox->firstItem(); item; item = item->next()) {
     static_cast<ListBoxText*>(item)->setColored(false);
     if(m_reordered) {
       Data::Field* field = static_cast<ListBoxText*>(item)->field();
       if(field) {
-        list.append(field);
+        fields.append(field);
       }
     }
   }
 
-  if(list.count() > 0) {
-    m_coll->reorderFields(list);
+  // if reordering fields, need to add ReadOnly fields since they were not shown
+  if(m_reordered) {
+    Data::FieldVec allFields = m_coll->fields();
+    for(Data::FieldVec::Iterator it = allFields.begin(); it != allFields.end(); ++it) {
+      if(it->type() == Data::Field::ReadOnly) {
+        fields.append(it);
+      }
+    }
   }
 
-  if(m_newFields.count() > 0 || m_copiedFields.count() > 0 || m_reordered) {
-    emit signalCollectionModified();
+  if(fields.count() > 0) {
+    Kernel::self()->reorderFields(fields);
   }
 
-  // now clear copied fields, they all got added to the collection, so don't delete them
+  // commit command group
+  Kernel::self()->endCommandGroup();
+
+  // now clear copied fields
   m_copiedFields.clear();
-  // clear new ones, too, do not delete them
+  // clear new ones, too
   m_newFields.clear();
 
   // the field type might have changed, so need to update the type combo list with possible values
+  QString currType = m_typeCombo->currentText();
   m_typeCombo->clear();
   m_typeCombo->insertStringList(newTypesAllowed(m_currentField->type()));
+  m_typeCombo->setCurrentItem(currType);
 
   enableButtonApply(false);
 }
@@ -396,42 +433,34 @@ void CollectionFieldsDialog::slotNew() {
 }
 
 void CollectionFieldsDialog::slotDelete() {
-  Data::Field* field = m_currentField;
-  if(!field) {
+  if(!m_currentField) {
     return;
   }
 
-  if(m_newFields.containsRef(field)) {
-    m_currentField = 0;
+  if(m_newFields.contains(m_currentField)) {
     m_fieldsBox->removeItem(m_fieldsBox->currentItem());
-    m_newFields.removeRef(field);
+    m_newFields.remove(m_currentField);
     m_fieldsBox->setSelected(m_fieldsBox->currentItem(), true);
     m_fieldsBox->ensureCurrentVisible();
-    delete field;
+    m_currentField = 0; // KShared gets auto-deleted
     return;
   }
 
-  QString str = i18n("<qt><p>Do you really want to delete the <em>%1</em> field? "
-                     "This action occurs immediately and can not be undone!</p></qt>").arg(field->title());
-  QString dontAsk = QString::fromLatin1("DeleteField");
-  int ret = KMessageBox::warningYesNo(this, str, i18n("Delete Field?"),
-                                      KStdGuiItem::yes(), KStdGuiItem::no(), dontAsk);
-  if(ret == KMessageBox::Yes) {
-    bool success = m_coll->deleteField(field);
-    if(success) {
-      m_currentField = 0;
-      emit signalCollectionModified();
-      m_fieldsBox->removeItem(m_fieldsBox->currentItem());
-      m_fieldsBox->setSelected(m_fieldsBox->currentItem(), true);
-      m_fieldsBox->ensureCurrentVisible();
-      enableButtonOK(true);
-    }
+  bool success = Kernel::self()->removeField(m_currentField);
+  if(success) {
+    m_currentField = 0;
+    emit signalCollectionModified();
+    m_fieldsBox->removeItem(m_fieldsBox->currentItem());
+    m_fieldsBox->setSelected(m_fieldsBox->currentItem(), true);
+    m_fieldsBox->ensureCurrentVisible();
+    enableButtonOK(true);
   }
 }
 
 void CollectionFieldsDialog::slotTypeChanged(const QString& type_) {
   Data::Field::Type type = Data::Field::Undef;
-  for(QMap<Data::Field::Type, QString>::ConstIterator it = m_typeMap.begin(); it != m_typeMap.end(); ++it) {
+  const Data::Field::FieldMap& fieldMap = Data::Field::typeMap();
+  for(Data::Field::FieldMap::ConstIterator it = fieldMap.begin(); it != fieldMap.end(); ++it) {
     if(it.data() == type_) {
       type = it.key();
       break;
@@ -442,10 +471,10 @@ void CollectionFieldsDialog::slotTypeChanged(const QString& type_) {
     type = Data::Field::Line;
   }
 
-  // only lists gets allowed values
+  // only choice types gets allowed values
   m_allowEdit->setEnabled(type == Data::Field::Choice);
 
-  // paragraphs and tables are their own category
+  // paragraphs, tables, and images are their own category
   bool isCategory = (type == Data::Field::Para || type == Data::Field::Table ||
                      type == Data::Field::Table2 || type == Data::Field::Image);
   m_catCombo->setEnabled(!isCategory);
@@ -467,7 +496,7 @@ void CollectionFieldsDialog::slotTypeChanged(const QString& type_) {
   m_complete->setEnabled(isText);
 
   // grouping is not possible with paragraphs or images
-  m_grouped->setEnabled(type_ != Data::Field::Para && type != Data::Field::Image);
+  m_grouped->setEnabled(type != Data::Field::Para && type != Data::Field::Image);
 }
 
 void CollectionFieldsDialog::slotHighlightedChanged(int index_) {
@@ -498,7 +527,7 @@ void CollectionFieldsDialog::slotHighlightedChanged(int index_) {
   }
 
   // need to get a pointer to the field with the new values to insert
-  Data::Field* field = item->field();
+  Data::FieldPtr field = item->field();
   if(!field) {
     kdDebug() << "CollectionFieldsDialog::slotHighlightedChanged() - no field found!" << endl;
     return;
@@ -508,14 +537,15 @@ void CollectionFieldsDialog::slotHighlightedChanged(int index_) {
 
   // type is limited to certain types, unless it's a new field
   m_typeCombo->clear();
-  if(m_newFields.containsRef(field)) {
+  if(m_newFields.contains(field)) {
     m_typeCombo->insertStringList(newTypesAllowed(Data::Field::Undef));
   } else {
     m_typeCombo->insertStringList(newTypesAllowed(field->type()));
   }
   // if the current name is not there, then this will change the list!
-  m_typeCombo->setCurrentText(m_typeMap[field->type()]);
-  slotTypeChanged(m_typeMap[field->type()]); // just setting the text doesn't emit the activated signal
+  const Data::Field::FieldMap& fieldMap = Data::Field::typeMap();
+  m_typeCombo->setCurrentText(fieldMap[field->type()]);
+  slotTypeChanged(fieldMap[field->type()]); // just setting the text doesn't emit the activated signal
 
   if(field->type() == Data::Field::Choice) {
     m_allowEdit->setText(field->allowed().join(QString::fromLatin1("; ")));
@@ -572,16 +602,9 @@ void CollectionFieldsDialog::updateField() {
   }
 
   // only update name if it's one of the new ones
-  if(m_newFields.containsRef(field)) {
+  if(m_newFields.contains(field)) {
     // name needs to be a valid XML element name
-    QString name = m_titleEdit->text().lower();
-    // remove white space
-    name.replace(QRegExp(QString::fromLatin1("\\s+")), QString::fromLatin1("-"));
-    // replace non word characters, but allow dashes
-    name.replace(QRegExp(QString::fromLatin1("[^\\w-]")), QString::null);
-    while(name.at(0).isDigit() || name.at(0) == '-') {
-      name = name.mid(1);
-    }
+    QString name = XML::elementName(m_titleEdit->text().lower());
     if(name.isEmpty()) { // might end up with empty string
       name = QString::fromLatin1("custom") + QString::number(m_newFields.count()+1);
     }
@@ -594,8 +617,8 @@ void CollectionFieldsDialog::updateField() {
   QString title = m_titleEdit->text().simplifyWhiteSpace();
   slotUpdateTitle(title);
 
-  QMap<Data::Field::Type, QString>::Iterator it;
-  for(it = m_typeMap.begin(); it != m_typeMap.end(); ++it) {
+  const Data::Field::FieldMap& fieldMap = Data::Field::typeMap();
+  for(Data::Field::FieldMap::ConstIterator it = fieldMap.begin(); it != fieldMap.end(); ++it) {
     if(it.data() == m_typeCombo->currentText()) {
       field->setType(it.key());
       break;
@@ -605,6 +628,17 @@ void CollectionFieldsDialog::updateField() {
   if(field->type() == Data::Field::Choice) {
     const QRegExp rx(QString::fromLatin1("\\s*;\\s*"));
     field->setAllowed(QStringList::split(rx, m_allowEdit->text()));
+    field->setProperty(QString::fromLatin1("minimum"), QString::null);
+    field->setProperty(QString::fromLatin1("maximum"), QString::null);
+  } else if(field->type() == Data::Field::Rating) {
+    QString v = field->property(QString::fromLatin1("minimum"));
+    if(v.isEmpty()) {
+      field->setProperty(QString::fromLatin1("minimum"), QString::number(1));
+    }
+    v = field->property(QString::fromLatin1("maximum"));
+    if(v.isEmpty()) {
+      field->setProperty(QString::fromLatin1("maximum"), QString::number(5));
+    }
   }
 
   if(field->isSingleCategory()) {
@@ -665,13 +699,13 @@ void CollectionFieldsDialog::slotModified() {
   static_cast<ListBoxText*>(m_fieldsBox->selectedItem())->setColored(true);
 
   // check if copy exists already
-  if(m_copiedFields.containsRef(m_currentField)) {
+  if(m_copiedFields.contains(m_currentField)) {
     return;
   }
 
   // or, check if is a new field, in which case no copy is needed
   // check if copy exists already
-  if(m_newFields.containsRef(m_currentField)) {
+  if(m_newFields.contains(m_currentField)) {
     return;
   }
 
@@ -710,7 +744,7 @@ void CollectionFieldsDialog::slotDefault() {
     return;
   }
 
-  QString caption = i18n("Revert Field Properties?");
+  QString caption = i18n("Revert Field Properties");
   QString text = i18n("<qt><p>Do you really want to revert the properties for the <em>%1</em> "
                       "field back to their default values?</p></qt>").arg(m_currentField->title());
   QString dontAsk = QString::fromLatin1("RevertFieldProperties");
@@ -723,8 +757,9 @@ void CollectionFieldsDialog::slotDefault() {
   m_updatingValues = true;
   m_titleEdit->setText(defaultField->title());
 
-  m_typeCombo->setCurrentText(m_typeMap[defaultField->type()]);
-  slotTypeChanged(m_typeMap[defaultField->type()]); // just setting the text doesn't emit the activated signal
+  const Data::Field::FieldMap& fieldMap = Data::Field::typeMap();
+  m_typeCombo->setCurrentText(fieldMap[defaultField->type()]);
+  slotTypeChanged(fieldMap[defaultField->type()]); // just setting the text doesn't emit the activated signal
 
   if(defaultField->type() == Data::Field::Choice) {
     m_allowEdit->setText(defaultField->allowed().join(QString::fromLatin1("; ")));
@@ -826,9 +861,9 @@ ListBoxText* CollectionFieldsDialog::findItem(const QListBox* box_, const Data::
   return 0;
 }
 
-void CollectionFieldsDialog::slotShowExtendedProperties() {
+bool CollectionFieldsDialog::slotShowExtendedProperties() {
   if(!m_currentField) {
-    return;
+    return false;
   }
 
   StringMapDialog* dlg = new StringMapDialog(m_currentField->propertyList(), this, "ExtendedPropertiesDialog", true);
@@ -836,9 +871,12 @@ void CollectionFieldsDialog::slotShowExtendedProperties() {
   dlg->setLabels(i18n("Property"), i18n("Value"));
   if(dlg->exec() == QDialog::Accepted) {
     m_currentField->setPropertyList(dlg->stringMap());
+    dlg->delayedDestruct();
     slotModified();
+    return true;
   }
   dlg->delayedDestruct();
+  return false;
 }
 
 bool CollectionFieldsDialog::checkValues() {
@@ -855,8 +893,14 @@ bool CollectionFieldsDialog::checkValues() {
   }
 
   const QString& category = m_currentField->category();
-  Data::FieldList list = m_coll->fieldsByCategory(category);
-  if(!list.isEmpty() && list.getFirst()->isSingleCategory() && list.getFirst()->name() != m_currentField->name()) {
+  if(category.isEmpty()) {
+    KMessageBox::sorry(this, i18n("<qt>The category may not be empty. Please enter a category.</qt>"));
+    m_catCombo->lineEdit()->selectAll();
+    return false;
+  }
+
+  Data::FieldVec fields = m_coll->fieldsByCategory(category);
+  if(!fields.isEmpty() && fields.begin()->isSingleCategory() && fields.begin()->name() != m_currentField->name()) {
     // can't have this category, cause it conflicts with a single-category field
     KMessageBox::sorry(this, i18n("<qt>A field may not be in the same category as a <em>Paragraph</em>, "
                                   "<em>Table</em> or <em>Image</em> field. Please enter a different category.</qt>"));
@@ -872,77 +916,101 @@ bool CollectionFieldsDialog::checkValues() {
     return false;
   }
 
+  // check for rating values outside bounds
+  if(m_currentField->type() == Data::Field::Rating) {
+    bool ok; // ok to ignore this here
+    int low = Tellico::toUInt(m_currentField->property(QString::fromLatin1("minimum")), &ok);
+    int high = Tellico::toUInt(m_currentField->property(QString::fromLatin1("maximum")), &ok);
+    while(low < 1 || low > 9 || high < 1 || high > 10 || low >= high) {
+      KMessageBox::sorry(this, i18n("The range for a rating field must be between 1 and 10, "
+                                    "and the lower bound must be less than the higher bound. "
+                                    "Please enter different low and high properties."));
+      if(slotShowExtendedProperties()) {
+        low = Tellico::toUInt(m_currentField->property(QString::fromLatin1("minimum")), &ok);
+        high = Tellico::toUInt(m_currentField->property(QString::fromLatin1("maximum")), &ok);
+      } else {
+        return false;
+      }
+    }
+  } else if(m_currentField->type() == Data::Field::Table) {
+    bool ok; // ok to ignore this here
+    int ncols = Tellico::toUInt(m_currentField->property(QString::fromLatin1("columns")), &ok);
+    // also enforced in GUI::TableFieldWidget
+    if(ncols > 5) {
+      KMessageBox::sorry(this, i18n("Tables are limited to a maximum of five columns."));
+      m_currentField->setProperty(QString::fromLatin1("columns"), QChar('5'));
+    }
+  }
+
   return true;
 }
 
 // only certain type changes are allowed
-QStringList CollectionFieldsDialog::newTypesAllowed(Data::Field::Type type_ /*=0*/) {
+QStringList CollectionFieldsDialog::newTypesAllowed(int type_ /*=0*/) {
   // Undef means return all
   if(type_ == Data::Field::Undef) {
     return Data::Field::typeTitles();
   }
 
+  const Data::Field::FieldMap& fieldMap = Data::Field::typeMap();
+
   QStringList newTypes;
   switch(type_) {
     case Data::Field::Line: // might not work if converted to a number or URL, but ok
     case Data::Field::Number:
     case Data::Field::URL:
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Para];
-      newTypes += m_typeMap[Data::Field::Number];
-      newTypes += m_typeMap[Data::Field::URL];
-      newTypes += m_typeMap[Data::Field::Table];
-      newTypes += m_typeMap[Data::Field::Table2];
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Para];
+      newTypes += fieldMap[Data::Field::Number];
+      newTypes += fieldMap[Data::Field::URL];
+      newTypes += fieldMap[Data::Field::Table];
       break;
 
     case Data::Field::Date:
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Date];
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Date];
       break;
 
     case Data::Field::Bool: // doesn't really make sense, but can't hurt
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Para];
-      newTypes += m_typeMap[Data::Field::Bool];
-      newTypes += m_typeMap[Data::Field::Number];
-      newTypes += m_typeMap[Data::Field::URL];
-      newTypes += m_typeMap[Data::Field::Table];
-      newTypes += m_typeMap[Data::Field::Table2];
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Para];
+      newTypes += fieldMap[Data::Field::Bool];
+      newTypes += fieldMap[Data::Field::Number];
+      newTypes += fieldMap[Data::Field::URL];
+      newTypes += fieldMap[Data::Field::Table];
       break;
 
     case Data::Field::Choice:
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Para];
-      newTypes += m_typeMap[Data::Field::Choice];
-      newTypes += m_typeMap[Data::Field::Number];
-      newTypes += m_typeMap[Data::Field::URL];
-      newTypes += m_typeMap[Data::Field::Table];
-      newTypes += m_typeMap[Data::Field::Table2];
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Para];
+      newTypes += fieldMap[Data::Field::Choice];
+      newTypes += fieldMap[Data::Field::Number];
+      newTypes += fieldMap[Data::Field::URL];
+      newTypes += fieldMap[Data::Field::Table];
+      newTypes += fieldMap[Data::Field::Rating];
       break;
 
-    case Data::Field::Table:
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Number];
-      newTypes += m_typeMap[Data::Field::Table];
-      newTypes += m_typeMap[Data::Field::Table2];
-      break;
-
-    case Data::Field::Table2: // not really a good idea since the "::" will be exposed, but allow it
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Number];
-      newTypes += m_typeMap[Data::Field::Table];
-      newTypes += m_typeMap[Data::Field::Table2];
+    case Data::Field::Table: // not really a good idea since the "::" will be exposed, but allow it
+    case Data::Field::Table2:
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Number];
+      newTypes += fieldMap[Data::Field::Table];
       break;
 
     case Data::Field::Para:
-      newTypes += m_typeMap[Data::Field::Line];
-      newTypes += m_typeMap[Data::Field::Para];
+      newTypes += fieldMap[Data::Field::Line];
+      newTypes += fieldMap[Data::Field::Para];
+      break;
+
+    case Data::Field::Rating:
+      newTypes += fieldMap[Data::Field::Choice];
+      newTypes += fieldMap[Data::Field::Rating];
       break;
 
     // these can never be changed
     case Data::Field::Image:
     case Data::Field::Dependent:
-      newTypes = m_typeMap[static_cast<Data::Field::Type>(type_)];
+      newTypes = fieldMap[static_cast<Data::Field::Type>(type_)];
       break;
 
     default:
