@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2005 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -11,32 +11,43 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <config.h>
+
 #include "fetchmanager.h"
 #include "configwidget.h"
-#include "../../config.h"
+#include "../tellico_kernel.h"
+
 #if AMAZON_SUPPORT
 #include "amazonfetcher.h"
 #endif
-#include "srufetcher.h"
 #if IMDB_SUPPORT
 #include "imdbfetcher.h"
 #endif
 #if HAVE_YAZ
 #include "z3950fetcher.h"
 #endif
-#include "../tellico_kernel.h"
+//#include "srufetcher.h"
+#include "entrezfetcher.h"
+#include "execexternalfetcher.h"
 
 #include <kglobal.h>
 #include <kconfig.h>
+#include <klocale.h>
 #include <kdebug.h>
 
 using Tellico::Fetch::Manager;
 Manager* Manager::s_self = 0;
-Tellico::Fetch::FetchKeyMap Manager::s_keyMap;
 
 Manager::Manager() : QObject(), m_count(0) {
-  m_fetchers.setAutoDelete(true);
   reloadFetchers();
+
+  m_keyMap.insert(FetchFirst, QString::null);
+  m_keyMap.insert(Title,      i18n("Title"));
+  m_keyMap.insert(Person,     i18n("Person"));
+  m_keyMap.insert(ISBN,       i18n("ISBN"));
+  m_keyMap.insert(Keyword,    i18n("Keyword"));
+  m_keyMap.insert(Raw,        i18n("Raw Query"));
+  m_keyMap.insert(FetchLast,  QString::null);
 }
 
 void Manager::reloadFetchers() {
@@ -90,47 +101,50 @@ void Manager::reloadFetchers() {
 //        m_fetchers.append(new SRUFetcher(this));
           break;
 
+        case Entrez:
+          f = new EntrezFetcher(this);
+          break;
+
+        case ExecExternal:
+          f = new ExecExternalFetcher(this);
+          break;
+
         case Unknown:
         default:
           continue;
       }
       if(f) {
         f->readConfig(config, group);
-        m_fetchers.append(f);
+        m_fetchers.push_back(f);
       }
     }
   } else { // add default sources
 #if AMAZON_SUPPORT
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::US, this));
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::UK, this));
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::DE, this));
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::JP, this));
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::FR, this));
-    m_fetchers.append(new AmazonFetcher(AmazonFetcher::CA, this));
+    m_fetchers.push_back(new AmazonFetcher(AmazonFetcher::US, this));
 #endif
 #if IMDB_SUPPORT
-    m_fetchers.append(new IMDBFetcher(this));
+    m_fetchers.push_back(new IMDBFetcher(this));
 #endif
 #if HAVE_YAZ
-    m_fetchers.append(Z3950Fetcher::libraryOfCongress(this));
+    m_fetchers.push_back(Z3950Fetcher::libraryOfCongress(this));
 #endif
   }
 
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    connect(it.current(), SIGNAL(signalStatus(const QString&)),
+  for(FetcherVec::ConstIterator it = m_fetchers.constBegin(); it != m_fetchers.constEnd(); ++it) {
+    connect(it.ptr(), SIGNAL(signalStatus(const QString&)),
             SIGNAL(signalStatus(const QString&)));
-    connect(it.current(), SIGNAL(signalResultFound(const Tellico::Fetch::SearchResult&)),
-            SIGNAL(signalResultFound(const Tellico::Fetch::SearchResult&)));
-    connect(it.current(), SIGNAL(signalDone(Tellico::Fetch::Fetcher*)),
+    connect(it.ptr(), SIGNAL(signalResultFound(Tellico::Fetch::SearchResult*)),
+            SIGNAL(signalResultFound(Tellico::Fetch::SearchResult*)));
+    connect(it.ptr(), SIGNAL(signalDone(Tellico::Fetch::Fetcher*)),
             SLOT(slotFetcherDone(Tellico::Fetch::Fetcher*)));
   }
 }
 
 QStringList Manager::sources() const {
   QStringList sources;
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    if(it.current()->canFetch(Kernel::self()->collection()->type())) {
-      sources << it.current()->source();
+  for(FetcherVec::ConstIterator it = m_fetchers.constBegin(); it != m_fetchers.constEnd(); ++it) {
+    if(it->canFetch(Kernel::self()->collectionType())) {
+      sources << it->source();
     }
   }
   return sources;
@@ -138,10 +152,10 @@ QStringList Manager::sources() const {
 
 QStringList Manager::keys(const QString& source_) const {
   // assume there's only one fetcher match
-  Fetcher* f = 0;
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    if(source_ == it.current()->source()) {
-      f = it.current();
+  const Fetcher* f = 0;
+  for(FetcherVec::ConstIterator it = m_fetchers.constBegin(); it != m_fetchers.constEnd(); ++it) {
+    if(source_ == it->source()) {
+      f = it.ptr();
       break;
     }
   }
@@ -167,10 +181,10 @@ void Manager::startSearch(const QString& source_, FetchKey key_, const QString&
   }
 
   // assume there's only one fetcher match
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    if(source_ == it.current()->source()) {
+  for(FetcherVec::Iterator it = m_fetchers.begin(); it != m_fetchers.end(); ++it) {
+    if(source_ == it->source()) {
       ++m_count; // Fetcher::search() might emit done(), so increment before calling search()
-      it.current()->search(key_, value_, multiple_);
+      it->search(key_, value_, multiple_);
       break;
     }
   }
@@ -178,9 +192,9 @@ void Manager::startSearch(const QString& source_, FetchKey key_, const QString&
 
 void Manager::stop() {
 //  kdDebug() << "Manager::stop()" << endl;
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    if(it.current()->isSearching()) {
-      it.current()->stop();
+  for(FetcherVec::Iterator it = m_fetchers.begin(); it != m_fetchers.end(); ++it) {
+    if(it->isSearching()) {
+      it->stop();
     }
   }
 #ifndef NDEBUG
@@ -200,23 +214,8 @@ void Manager::slotFetcherDone(Tellico::Fetch::Fetcher*) {
   }
 }
 
-// static
-void Manager::initMap() {
-  s_keyMap.clear();
-  s_keyMap.insert(FetchFirst, QString::null);
-  s_keyMap.insert(Title,      i18n("Title"));
-  s_keyMap.insert(Person,     i18n("Person"));
-  s_keyMap.insert(ISBN,       i18n("ISBN"));
-  s_keyMap.insert(Keyword,    i18n("Keyword"));
-  s_keyMap.insert(Raw,        i18n("Raw Query"));
-  s_keyMap.insert(FetchLast,  QString::null);
-}
-
-Tellico::Fetch::FetchKey Manager::fetchKey(const QString& key_) {
-  if(s_keyMap.isEmpty()) {
-    initMap();
-  }
-  for(FetchKeyMap::Iterator it = s_keyMap.begin(); it != s_keyMap.end(); ++it) {
+Tellico::Fetch::FetchKey Manager::fetchKey(const QString& key_) const {
+  for(FetchKeyMap::ConstIterator it = m_keyMap.begin(); it != m_keyMap.end(); ++it) {
     if(it.data() == key_) {
       return it.key();
     }
@@ -224,22 +223,20 @@ Tellico::Fetch::FetchKey Manager::fetchKey(const QString& key_) {
   return FetchFirst;
 }
 
-QString Manager::fetchKeyString(FetchKey key_) {
-  if(s_keyMap.isEmpty()) {
-    initMap();
-  }
-  return s_keyMap[key_];
+const QString& Manager::fetchKeyString(FetchKey key_) const {
+  return m_keyMap[key_];
 }
 
 bool Manager::canFetch() const {
-  for(FetcherListIterator it(m_fetchers); it.current(); ++it) {
-    if(it.current()->canFetch(Kernel::self()->collection()->type())) {
+  for(FetcherVec::ConstIterator it = m_fetchers.constBegin(); it != m_fetchers.constEnd(); ++it) {
+    if(it->canFetch(Kernel::self()->collectionType())) {
       return true;
     }
   }
   return false;
 }
 
+// static
 // called when creating a new fetcher
 Tellico::Fetch::ConfigWidget* Manager::configWidget(Type type_, QWidget* parent_) {
   switch(type_) {
@@ -247,8 +244,6 @@ Tellico::Fetch::ConfigWidget* Manager::configWidget(Type type_, QWidget* parent_
     case Amazon:
       return new AmazonFetcher::ConfigWidget(parent_);
 #endif
-    case SRU:
-      return new SRUFetcher::ConfigWidget(parent_);
 #if IMDB_SUPPORT
     case IMDB:
       return new IMDBFetcher::ConfigWidget(parent_);
@@ -257,6 +252,12 @@ Tellico::Fetch::ConfigWidget* Manager::configWidget(Type type_, QWidget* parent_
     case Z3950:
       return new Z3950Fetcher::ConfigWidget(parent_);
 #endif
+//    case SRU:
+//      return new SRUFetcher::ConfigWidget(parent_);
+    case Entrez:
+      return new EntrezFetcher::ConfigWidget(parent_);
+    case ExecExternal:
+      return new ExecExternalFetcher::ConfigWidget(parent_);
     default:
       kdDebug() << "Fetch::Manager::configWidget() - no widget defined for type = " << type_ << endl;
       return 0;
@@ -277,6 +278,8 @@ Tellico::Fetch::FetchMap Manager::sourceMap() {
 #endif
   // SRU not ready yet
 //  map.insert(SRU, i18n("SRU"));
+  map.insert(Entrez, i18n("Entrez Database"));
+  map.insert(ExecExternal, i18n("External Application"));
   return map;
 }
 
