@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2005 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -18,3 +18,10 @@
 const char* Tellico::errorLoad = I18N_NOOP("Tellico is unable to load the file - %1.");
 const char* Tellico::errorWrite = I18N_NOOP("Tellico is unable to write the file - %1.");
 const char* Tellico::errorUpload = I18N_NOOP("Tellico is unable to upload the file - %1.");
+const char* Tellico::errorAppendType = I18N_NOOP("Only collections with the same type of entries as "
+                                                 "the current one can be appended. No changes are being "
+                                                 "made to the current collection.");
+const char* Tellico::errorMergeType = I18N_NOOP("Only collections with the same type of entries as "
+                                                "the current one can be merged. No changes are being "
+                                                "made to the current collection.");
+const char* Tellico::errorImageLoad = I18N_NOOP("Tellico is unable to load an image from the file - %1.");
