@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2005 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -11,14 +11,16 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #include "bibtexexporter.h"
-#include "../collection.h"
+#include "bibtexhandler.h"
+#include "../document.h"
 #include "../collections/bibtexcollection.h"
 #include "../latin1literal.h"
+#include "../filehandler.h"
+#include "../stringset.h"
+
+#include <config.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -35,7 +37,7 @@
 
 using Tellico::Export::BibtexExporter;
 
-BibtexExporter::BibtexExporter(const Data::Collection* coll_) : Tellico::Export::TextExporter(coll_),
+BibtexExporter::BibtexExporter() : Tellico::Export::Exporter(),
    m_expandMacros(false),
    m_packageURL(true),
    m_skipEmptyKeys(false),
@@ -47,7 +49,149 @@ QString BibtexExporter::formatString() const {
 }
 
 QString BibtexExporter::fileFilter() const {
-  return i18n("*.bib|Bibtex files (*.bib)") + QChar('\n') + i18n("*|All files");
+  return i18n("*.bib|Bibtex Files (*.bib)") + QChar('\n') + i18n("*|All Files");
+}
+
+bool BibtexExporter::exec() {
+  const Data::Collection* c = Data::Document::self()->collection();
+  if(!c || c->type() != Data::Collection::Bibtex) {
+    return false;
+  }
+  const Data::BibtexCollection* coll = static_cast<const Data::BibtexCollection*>(c);
+
+// there are some special attributes
+// the entry-type specifies the entry type - book, inproceedings, whatever
+  QString typeField;
+// the key specifies the cite-key
+  QString keyField;
+// the crossref bibtex field can reference another entry
+  QString crossRefField;
+  bool hasCrossRefs = false;
+
+  const QString bibtex = QString::fromLatin1("bibtex");
+// keep a list of all the 'ordinary' fields to iterate through later
+  Data::FieldVec fields;
+  Data::FieldVec vec = coll->fields();
+  for(Data::FieldVec::Iterator it = vec.begin(); it != vec.end(); ++it) {
+    QString bibtexField = it->property(bibtex);
+    if(bibtexField == Latin1Literal("entry-type")) {
+      typeField = it->name();
+    } else if(bibtexField == Latin1Literal("key")) {
+      keyField = it->name();
+    } else if(bibtexField == Latin1Literal("crossref")) {
+      fields.append(it); // still output crossref field
+      crossRefField = it->name();
+      hasCrossRefs = true;
+    } else if(!bibtexField.isEmpty()) {
+      fields.append(it);
+    }
+  }
+
+
+  if(typeField.isEmpty() || keyField.isEmpty()) {
+    kdWarning() << "BibtexExporter::exec() - the collection must have fields defining "
+                   "the entry-type and the key of the entry" << endl;
+    return false;
+  }
+  if(fields.isEmpty()) {
+    kdWarning() << "BibtexExporter::exec() - no bibtex field mapping exists in the collection." << endl;
+    return false;
+  }
+
+  QString text = QString::fromLatin1("@comment{Generated by Tellico ")
+               + QString::fromLatin1(VERSION)
+               + QString::fromLatin1("}\n");
+
+  if(!coll->preamble().isEmpty()) {
+    text += QString::fromLatin1("@preamble{") + coll->preamble() + QString::fromLatin1("}\n");
+  }
+
+  const QStringList macros = coll->macroList().keys();
+  if(!m_expandMacros) {
+    QMap<QString, QString>::ConstIterator macroIt;
+    for(macroIt = coll->macroList().constBegin(); macroIt != coll->macroList().constEnd(); ++macroIt) {
+      if(!macroIt.data().isEmpty()) {
+        text += QString::fromLatin1("@string{")
+                + macroIt.key()
+                + QString::fromLatin1("=")
+                + BibtexHandler::exportText(macroIt.data(), macros)
+                + QString::fromLatin1("}\n");
+      }
+    }
+  }
+
+  // if anything is crossref'd, we have to do an initial scan through the
+  // whole collection first
+  StringSet crossRefKeys;
+  if(hasCrossRefs) {
+    QString tmp;
+    for(Data::EntryVec::ConstIterator entryIt = entries().begin(); entryIt != entries().end(); ++entryIt) {
+      tmp = entryIt->field(crossRefField);
+      if(!tmp.isEmpty()) {
+        crossRefKeys.add(tmp);
+      }
+    }
+  }
+
+
+  StringSet usedKeys;
+  Data::ConstEntryVec crossRefs;
+  QString type, key, newKey, value;
+  for(Data::EntryVec::ConstIterator entryIt = entries().begin(); entryIt != entries().end(); ++entryIt) {
+    type = entryIt->field(typeField);
+    if(type.isEmpty()) {
+      kdWarning() << "BibtexExporter::text() - the entry for '" << entryIt->title()
+                  << "' has no entry-type, skipping it!" << endl;
+      continue;
+    }
+
+    key = entryIt->field(keyField);
+    if(key.isEmpty()) {
+      if(m_skipEmptyKeys) {
+        continue;
+      }
+      key = BibtexHandler::bibtexKey(entryIt);
+    } else {
+      // check crossrefs, only counts for non-empty keys
+      // if this entry is crossref'd, add it to the list, and skip it
+      if(hasCrossRefs && crossRefKeys.has(key)) {
+        crossRefs.append(entryIt);
+        continue;
+      }
+    }
+
+    newKey = key;
+    uint c = 'a';
+    while(usedKeys.has(newKey)) {
+      // duplicate found!
+      newKey = key + c;
+      ++c;
+    }
+    key = newKey;
+    usedKeys.add(key);
+
+    writeEntryText(text, fields, *entryIt, type, key);
+  }
+
+  // now write out crossrefs
+  for(Data::ConstEntryVec::ConstIterator entryIt = crossRefs.constBegin(); entryIt != crossRefs.constEnd(); ++entryIt) {
+    // no need to check type
+
+    key = entryIt->field(keyField);
+    newKey = key;
+    uint c = 'a';
+    while(usedKeys.has(newKey)) {
+      // duplicate found!
+      newKey = key + c;
+      ++c;
+    }
+    key = newKey;
+    usedKeys.add(key);
+
+    writeEntryText(text, fields, *entryIt, entryIt->field(typeField), key);
+  }
+
+  return FileHandler::writeTextURL(url(), text, options() & ExportUTF8, options() & Export::ExportForce);
 }
 
 QWidget* BibtexExporter::widget(QWidget* parent_, const char* name_/*=0*/) {
@@ -126,121 +270,40 @@ void BibtexExporter::saveOptions(KConfig* config_) {
   }
 }
 
-QString BibtexExporter::text(bool formatFields_, bool) {
-// there are some special attributes
-// the entry-type specifies the entry type - book, inproceedings, whatever
-  QString typeField;
-// the key specifies the cite-key
-  QString keyField;
-// keep a list of all the 'ordinary' fields to iterate through later
-  Data::FieldList list;
-  for(Data::FieldListIterator it(collection()->fieldList()); it.current(); ++it) {
-    QString bibtex = it.current()->property(QString::fromLatin1("bibtex"));
-    if(bibtex == Latin1Literal("entry-type")) {
-      typeField = it.current()->name();
-    } else if(bibtex == Latin1Literal("key")) {
-      keyField = it.current()->name();
-    } else if(!bibtex.isEmpty()) {
-      list.append(it.current());
-    }
-  }
+void BibtexExporter::writeEntryText(QString& text_, const Data::FieldVec& fields_, const Data::Entry& entry_,
+                                    const QString& type_, const QString& key_) {
+  const QStringList macros = static_cast<const Data::BibtexCollection*>(Data::Document::self()->collection())->macroList().keys();
+  const QString bibtex = QString::fromLatin1("bibtex");
 
-  if(typeField.isEmpty() || keyField.isEmpty()) {
-    kdWarning() << "BibtexExporter::text() - the collection must have fields defining "
-                   "the entry-type and the key of the entry" << endl;
-    return QString::null;
-  }
-  if(list.isEmpty()) {
-    kdWarning() << "BibtexExporter::text() - no bibtex field mapping exists in the collection." << endl;
-    return QString::null;
-  }
+  text_ += '@' + type_ + '{' + key_;
 
-  QString text = QString::fromLatin1("@comment{Generated by Tellico ");
-  text += QString::fromLatin1(VERSION);
-  text += QString::fromLatin1("}\n");
-
-  const Data::BibtexCollection* c = dynamic_cast<const Data::BibtexCollection*>(collection());
-  QStringList macros;
-  if(c) {
-    QString pre = c->preamble();
-    if(!pre.isEmpty()) {
-      text += QString::fromLatin1("@preamble{") + pre + QString::fromLatin1("}\n");
-    }
-
-    if(!m_expandMacros) {
-      macros = c->macroList().keys();
-      QMap<QString, QString>::ConstIterator macroIt;
-      for(macroIt = c->macroList().begin(); macroIt != c->macroList().end(); ++macroIt) {
-        if(!macroIt.data().isEmpty()) {
-          text += QString::fromLatin1("@string{")
-                  + macroIt.key()
-                  + QString::fromLatin1("=")
-                  + BibtexHandler::exportText(macroIt.data(), macros)
-                  + QString::fromLatin1("}\n");
-        }
-      }
-    }
-  }
-
-  // use a dict for fast random access to keep track of the bibtex cite keys
-  QDict<int> citeKeyDict;
-  Data::FieldListIterator fIt(list);
-  for(Data::EntryListIterator entryIt(entryList()); entryIt.current(); ++entryIt) {
-    QString type = entryIt.current()->field(typeField);
-    if(type.isEmpty()) {
-      kdWarning() << "BibtexExporter::text() - the entry for '" << entryIt.current()->title()
-                  << "' has no entry-type, skipping it!" << endl;
+  QString value;
+  Data::FieldVec::ConstIterator fIt, end = fields_.constEnd();
+  bool format = options() & Export::ExportFormatted;
+  for(fIt = fields_.constBegin(); fIt != end; ++fIt) {
+    value = entry_.field(fIt->name(), format);
+    if(value.isEmpty()) {
       continue;
     }
 
-    QString key = entryIt.current()->field(keyField);
-    if(key.isEmpty()) {
-      if(m_skipEmptyKeys) {
-        continue;
-      }
-      key = BibtexHandler::bibtexKey(entryIt.current());
+    // If the entry is formatted as a name and allows multiple values
+    // insert "and" in between them (e.g. author and editor)
+    if(fIt->formatFlag() == Data::Field::FormatName
+       && fIt->flags() & Data::Field::AllowMultiple) {
+      value.replace(Data::Field::delimiter(), QString::fromLatin1(" and "));
     }
-    QString newKey = key;
-    uint c = 'a';
-    while(citeKeyDict[newKey]) {
-      // duplicate found!
-      newKey = key + c;
-      ++c;
-    }
-    key = newKey;
-    citeKeyDict.insert(key, reinterpret_cast<const int *>(1));
 
-    text += QString::fromLatin1("@") + type + QString::fromLatin1("{");
-    text += key;
-
-    QString value;
-    for(fIt.toFirst(); fIt.current(); ++fIt) {
-      if(formatFields_) {
-        value = entryIt.current()->formattedField(fIt.current()->name());
-      } else {
-        value = entryIt.current()->field(fIt.current()->name());
-      }
-      // If the entry is formatted as a name and allow multiple values
-      // insert "and" in between them (e.g. author and editor)
-      if(fIt.current()->formatFlag() == Data::Field::FormatName
-         && fIt.current()->flags() & Data::Field::AllowMultiple) {
-        value.replace(Data::Field::delimiter(), QString::fromLatin1(" and "));
-      }
-      if(!value.isEmpty()) {
-        if(m_packageURL && fIt.current()->type() == Data::Field::URL) {
-          value = QString::fromLatin1("\\url{") + value + QString::fromLatin1("}");
-        } else if(fIt.current()->type() != Data::Field::Number) {
-        // numbers aren't escaped, nor will they have macros
-          // if m_expandMacros is true, then macros is empty, so this is ok even then
-          value = BibtexHandler::exportText(value, macros);
-        }
-        text += QString::fromLatin1(",\n  ");
-        text += fIt.current()->property(QString::fromLatin1("bibtex"));
-        text += QString::fromLatin1(" = ");
-        text += value;
-      }
+    if(m_packageURL && fIt->type() == Data::Field::URL) {
+      value = QString::fromLatin1("\\url{") + value + QString::fromLatin1("}");
+    } else if(fIt->type() != Data::Field::Number) {
+      // numbers aren't escaped, nor will they have macros
+      // if m_expandMacros is true, then macros is empty, so this is ok even then
+      value = BibtexHandler::exportText(value, macros);
     }
-    text += QString::fromLatin1("\n}\n");
+    text_ += QString::fromLatin1(",\n  ")
+           + fIt->property(bibtex)
+           + QString::fromLatin1(" = ")
+           + value;
   }
-  return text;
+  text_ += QString::fromLatin1("\n}\n");
 }
