@@ -1,5 +1,5 @@
 /***************************************************************************
-    copyright            : (C) 2003-2004 by Robby Stephenson
+    copyright            : (C) 2003-2005 by Robby Stephenson
     email                : robby@periapsis.org
  ***************************************************************************/
 
@@ -16,23 +16,23 @@
 
 class KURLRequester;
 
-#include "textexporter.h"
+#include "exporter.h"
 
 namespace Tellico {
   namespace Export {
 
 /**
  * @author Robby Stephenson
- * @version $Id: xsltexporter.h 862 2004-09-15 01:49:51Z robby $
  */
-class XSLTExporter : public TextExporter {
+class XSLTExporter : public Exporter {
 public:
-  XSLTExporter(const Data::Collection* coll);
+  XSLTExporter();
+
+  virtual bool exec();
+  virtual QString formatString() const;
+  virtual QString fileFilter() const;
 
   virtual QWidget* widget(QWidget* parent, const char* name=0);
-  virtual QString formatString() const;
-  virtual QString text(bool format, bool encodeUTF8);
-  virtual QString fileFilter() const;
   virtual void readOptions(KConfig*) {}
   virtual void saveOptions(KConfig*) {}
 
